# -*- coding: utf-8 -*-
#
# This file is part of fedrepo_req.
# Copyright © 2017 Red Hat, Inc.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
"""
The fedrepo-req script
"""
from __future__ import absolute_import
import click
import json
from fedrepo_req import NAMESPACE_CHOICES, MONITOR_CHOICES
from fedrepo_req import BUGZILLA_CLIENT, BRANCH_DEFAULT
from fedrepo_req.utils import (
    sla_list_to_dict, verify_slas, is_valid_standard_branch)
from fedrepo_req.exceptions import ValidationError
import fedrepo_req.pagure

SLA_HELP = ('The SLs tied to the branch. This must be in the format of '
            '"sl_name:2017-12-25"')


@click.command()
@click.option('--description', '-d', help='The package\'s description.')
@click.option('--monitor', '-m', type=click.Choice(MONITOR_CHOICES),
              default=MONITOR_CHOICES[2],
              help='Monitoring type for the package.')
@click.option('--namespace', '-n', type=click.Choice(NAMESPACE_CHOICES),
              default='rpms', help='The repo\'s namespace.')
@click.option('--summary', '-s',
              help='Override the package\'s summary from the ticket.')
@click.option('--ticket', '-t', help='The package request\'s ticket number.',
              prompt=True)
@click.option('--upstreamurl', '-u', help='The package\'s upstream URL.')
@click.option('--sl', multiple=True, help=SLA_HELP)
@click.argument('repo')
@click.argument('branch', default=BRANCH_DEFAULT)
def cli(branch, repo, sl, upstreamurl, ticket, summary, namespace, monitor,
        description):
    """
    The entry point of the Click application
    """
    # This name makes more sense for the contents of sl, where as --sl makes
    # more sense from the cli perspective
    slas = sl
    branch = branch.strip()
    repo = repo.strip()

    if namespace in ['modules', 'test-modules'] and '-' in branch:
        raise click.ClickException('Dashes are not allowed in module branches')

    # If SLAs are not provided and it isn't a standard branch, then raise an
    # error
    if not slas and not is_valid_standard_branch(branch):
        raise click.ClickException(
            'You must provide SLs for non-standard branches')
    # If SLAs were provided, no matter what kind of branch it is, verify them
    elif slas:
        try:
            sla_dict = sla_list_to_dict(list(slas))
            verify_slas(branch, sla_dict)
        except ValidationError as error:
            raise click.ClickException(str(error))

    try:
        bug = BUGZILLA_CLIENT.get_valid_review_bug(
            ticket, repo, require_auth=False, namespace=namespace)
    except ValidationError as error:
        raise click.ClickException(str(error))

    summary_from_bug = bug.summary.split(' - ', 1)[1].strip()

    ticket_body = {
        'action': 'new_repo',
        'branch': branch,
        'bug_id': ticket,
        'description': description or '',
        'monitor': monitor,
        'namespace': namespace,
        'repo': repo,
        'summary': summary or summary_from_bug,
        'upstreamurl': upstreamurl or ''
    }
    if slas:
        ticket_body['sls'] = sla_dict
    ticket_body = json.dumps(ticket_body, indent=True)
    ticket_body = '```\n{0}\n```'.format(ticket_body)
    ticket_title = 'New Repo for "{0}/{1}"'.format(namespace, repo)
    click.echo(fedrepo_req.pagure.new_issue(ticket_title, ticket_body))


if __name__ == '__main__':
    cli()
